﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public class NPCPatrol : MonoBehaviour
{
    //Dictates wheter the agent waits on each node.
    [SerializeField]
    bool _patrolWaiting;

    //The total time AI waits on each node.
    [SerializeField]
    float _totalWaitTime = 3f;

    //The chance of changing direction.
    [SerializeField]
    float _switchProbrability = 0.2f;

    //List of all nodes
    [SerializeField]
    List<Waypoint> _patrolPoints;

    //Private variables for base behaviour.
    NavMeshAgent _navMeshAgent;
    int _currentPatrolIndex;
    bool _travelling;
    bool _waiting;
    bool _patrolForward;
    float _waitTimer;


    public void Start()
    {
        _navMeshAgent = this.GetComponent<NavMeshAgent>();

        if (_navMeshAgent == null)
        {
            Debug.LogError("The NavMeshAgent component is not attached to " + gameObject.name);
        }
        else
        {
            if(_patrolPoints != null && _patrolPoints.Count >= 2)
            {
                _currentPatrolIndex = 0;
                SetDestination();
            }
            else
            {
                Debug.Log("Insufficient patrol points for basic patrolling behaviour.");
            }
                  
        }

    }

    public void Update()
    {

        //Check if AI is too close to patrol point
        if (_travelling && _navMeshAgent.remainingDistance <= 1.0f)
        {
            _travelling = false;


            //If AI is going to wait, then wait
            if (_patrolWaiting)
            {
                _waiting = true;
                _waitTimer = 0f;
            }
            else
            {
                ChangePatrolPoint();
                SetDestination();
            }
        }
        if (_waiting)
        {
            _waitTimer += Time.deltaTime;
            if (_waitTimer >= _totalWaitTime)
            {
                _waiting = false;

                ChangePatrolPoint();
                SetDestination();
            }
        }

    }


private void SetDestination()
{
    if(_patrolPoints != null)
    {
        Vector3 targetVector = _patrolPoints[_currentPatrolIndex].transform.position;
        _navMeshAgent.SetDestination(targetVector);
        _travelling = true;
    }
}

private void ChangePatrolPoint()
{
    if(UnityEngine.Random.Range(0f, 1f) <=_switchProbrability)
    {
        _patrolForward = !_patrolForward;
    }

    if(_patrolForward)
    {
        _currentPatrolIndex = (_currentPatrolIndex + 1) % _patrolPoints.Count;
    }
    else
    {
        if(--_currentPatrolIndex < 0)
        {
            _currentPatrolIndex = _patrolPoints.Count - 1;
        }
    }
  }
}