﻿using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class Projectile : MonoBehaviour
{
    public float damage = 10f;

    // bullet 
    public GameObject bullet;

    // bullet force
    public float shootForce, upwardForce;

    // Gun stats 
    public float timeBetweenShooting, spread, reloadTime, timeBetweenShots;
    public int magazineSize, bulletsPerTap;
    public bool allowButtonHold;

    private int bulletsLeft, bulletsShot;

    // Recoil
    public Rigidbody playerRb;
    public float recoilForce;

    // bools 
    bool shooting, readyToShoot, reloading;

    // Reference 
    public Camera fpsCam;
    public Transform attackPoint;

    // Graphics 
    public GameObject muzzleFlash;
    //public TextMeshProUGUI ammunitionDisplay;
    public Text ammunitionDisplay;

    // Bug fixing 
    public bool allowInvoke = true;

    private void Awake()
    {
        // make sure the magazine size is full 
        bulletsLeft = magazineSize;
        readyToShoot = true;
    }

    private void Update()
    {
        MyInput();

        // Set ammo display, if it exists 
        if (ammunitionDisplay != null)
            ammunitionDisplay.text = "Ammo: " + bulletsLeft / bulletsPerTap + "/" + magazineSize / bulletsPerTap;
    }

    private void MyInput()
    {
        // Check if allowed to hold down button and take corresponding input
        if (allowButtonHold) shooting = Input.GetKey(KeyCode.Mouse0);
        else shooting = Input.GetKeyDown(KeyCode.Mouse0);

        // Reloading 
        if (Input.GetKeyDown(KeyCode.R) && bulletsLeft < magazineSize && !reloading) Reload();

        // Reload automatically when trying to shoot without ammo
        if (readyToShoot && shooting && !reloading && bulletsLeft <= 0) Reload();

        // Important code 
        // This code destroys the gun object after magazine size has gotten to zero
        //if (readyToShoot && shooting && !reloading && bulletsLeft <= 0)
        // {
        //        Destroy(GameObject.FindGameObjectWithTag("Gun"), 0.1f);
        //  }


        // Shooting 
        if (readyToShoot && shooting && !reloading && bulletsLeft > 0)
        {
            // Set bullet shot to 0
            bulletsShot = 0;

            Shoot();
        }
        // Destroys the clones of the BigExplosionEffect
        Destroy(GameObject.Find("BigExplosionEffect(Clone)"), 1f);
    }

    private void Shoot()
    {
        readyToShoot = false;

        // Find the exact hit position using a raycast 
        Ray ray = fpsCam.ViewportPointToRay(new Vector3(0.5f, 0.5f, 0)); // Just a ray through the middle of your screen 
        RaycastHit hit;

        // Check if ray hits something 
        Vector3 targetPoint;
        if (Physics.Raycast(ray, out hit))
        {
            targetPoint = hit.point;

            Target target = hit.transform.GetComponent<Target>();

            if (target != null)
            {
                target.TakeDamage(damage);
            }

            OtherTarget other = hit.transform.GetComponent<OtherTarget>();

            if (other != null)
            {
                other.TakeDamage(damage);
            }
        }
        else
            targetPoint = ray.GetPoint(75); // Just a point far away from the player

        // Calculate direction from attackPoint to targetPoint
        Vector3 directionWithoutSpread = targetPoint - attackPoint.position;

        // Calculate spread 
        float x = Random.Range(-spread, spread);
        float y = Random.Range(-spread, spread);

        // Calculate the new direction with spread
        Vector3 directionWithSpread = directionWithoutSpread + new Vector3(x, y, 0);

        // Instantiate bullet/Projectile
        GameObject currentBullet = Instantiate(bullet, attackPoint.position, Quaternion.identity);

        // Rotate bullet to shoot direction 
        currentBullet.transform.forward = directionWithSpread.normalized;

        // Add force to bullet 
        currentBullet.GetComponent<Rigidbody>().AddForce(directionWithSpread.normalized * shootForce, ForceMode.Impulse);
        currentBullet.GetComponent<Rigidbody>().AddForce(fpsCam.transform.up * upwardForce, ForceMode.Impulse);

        // Instantiate muzzle flash, if you have one
        if (muzzleFlash != null)
            Instantiate(muzzleFlash, attackPoint.position, Quaternion.identity);

        bulletsLeft--;
        bulletsShot++;

        // Invoke resetShot function (if not already invoked) with your timeBetweenShooting
        if (allowInvoke)
        {
            Invoke("ResetShot", timeBetweenShooting);
            allowInvoke = false;

            // Add recoil to player
            playerRb.AddForce(-directionWithSpread.normalized * recoilForce, ForceMode.Impulse);
        }

        // if more than one bulletsPerTap make sure to repeat shoot function 
        if (bulletsShot < bulletsPerTap && bulletsLeft > 0)
            Invoke("Shoot", timeBetweenShots);

        // Destroys clones of the MuzzleFlashEffect
        Destroy(GameObject.Find("MuzzleFlashEffect(Clone)"), 0.1f);
    }

    private void ResetShot()
    {
        // Allow shooting and invoking again
        readyToShoot = true;
        allowInvoke = true;
    }

    private void Reload()
    {
        reloading = true;
        Invoke("ReloadFinished", reloadTime);
    }

    private void ReloadFinished()
    {
        bulletsLeft = magazineSize;
        reloading = false;
    }
}
